﻿using UnityEngine;
using System.Collections;

public class HandlePlayer : MonoBehaviour
{
    IngameSceneLogic ingameLogicComponent = null;

    public GameObject PlayerAvatarBody;
    public GameObject PlayerAvatarWings;

    float playerLeftBodyAngleDeg = -135.0f;
    float playerRightBodyAngleDeg = -225.0f;
    float playerCurrentBodyYawAngleDeg;
    float playerRequestedBodyYawAngleDeg;
    float playerBodyTurnSpeedDegPerSec = 240.0f;

    float playerCenterWingsPitchAngleDeg = 0.0f;
    float playerUpWingsPitchAngleDeg = -45.0f;
    float playerDownWingsPitchAngleDeg = 45.0f;
    float playerCurrentWingsPitchAngleDeg;
    float playerRequestedWingsPitchAngleDeg;
    float playerWingsPitchSpeedDegPerSec = 120.0f;

    float inputDeadZone = 0.2f;

    bool isPlayerMoving = false;
    float playerSpeed = 0.1f;
    bool playerMoveLeft = false;
    bool playerMoveRight = false;
    bool playerMoveUp = false;
    bool playerMoveDown = false;
    float playerTargetX;
    float playerTargetY;

    void Start()
    {
        GameObject ingameSceneLogicAnchor = GameObject.Find("IngameSceneAnchor");
        ingameLogicComponent = ingameSceneLogicAnchor.GetComponent<IngameSceneLogic>();

        playerCurrentBodyYawAngleDeg = playerLeftBodyAngleDeg;
        playerRequestedBodyYawAngleDeg = playerLeftBodyAngleDeg;

        Vector3 playerRotation = PlayerAvatarBody.transform.eulerAngles;
        playerRotation.y = playerCurrentBodyYawAngleDeg;
        PlayerAvatarBody.transform.eulerAngles = playerRotation;

        playerCurrentWingsPitchAngleDeg = 0.0f;
        playerRequestedWingsPitchAngleDeg = 0.0f;
        Vector3 playerWingsRotation = PlayerAvatarWings.transform.eulerAngles;
        playerWingsRotation.x = playerCurrentWingsPitchAngleDeg;
        PlayerAvatarWings.transform.eulerAngles = playerWingsRotation;
    }

    void Update()
    {
        if (ingameLogicComponent && ingameLogicComponent.IngameReady && ingameLogicComponent.AllowPlayerInput)
        {
            float horzValue = Input.GetAxis("Horizontal");
            float vertValue = Input.GetAxis("Vertical");

            HandlePlayerMovement(horzValue, vertValue);

            HandlePlayerBody(horzValue);
            HandlePlayerWings(vertValue);
        }
    }

    void HandlePlayerBody(float horzValue)
    {
        if (horzValue <= -inputDeadZone && playerMoveLeft)
        {
            playerRequestedBodyYawAngleDeg = playerLeftBodyAngleDeg;
        }
        else if(horzValue >= inputDeadZone && playerMoveRight)
        {
            playerRequestedBodyYawAngleDeg = playerRightBodyAngleDeg;
        }

        if(playerCurrentBodyYawAngleDeg > playerRequestedBodyYawAngleDeg)
        {
            playerCurrentBodyYawAngleDeg -= playerBodyTurnSpeedDegPerSec * Time.deltaTime;

            if (playerCurrentBodyYawAngleDeg < playerRequestedBodyYawAngleDeg)
            {
                playerCurrentBodyYawAngleDeg = playerRequestedBodyYawAngleDeg;
            }

            Vector3 playerRotation = PlayerAvatarBody.transform.eulerAngles;
            playerRotation.y = playerCurrentBodyYawAngleDeg;
            PlayerAvatarBody.transform.eulerAngles = playerRotation;
        }
        else if(playerCurrentBodyYawAngleDeg < playerRequestedBodyYawAngleDeg)
        {
            playerCurrentBodyYawAngleDeg += playerBodyTurnSpeedDegPerSec * Time.deltaTime;

            if (playerCurrentBodyYawAngleDeg > playerRequestedBodyYawAngleDeg)
            {
                playerCurrentBodyYawAngleDeg = playerRequestedBodyYawAngleDeg;
            }

            Vector3 playerRotation = PlayerAvatarBody.transform.eulerAngles;
            playerRotation.y = playerCurrentBodyYawAngleDeg;
            PlayerAvatarBody.transform.eulerAngles = playerRotation;
        }
    }

    void HandlePlayerWings(float vertValue)
    {
        if (vertValue <= -inputDeadZone && playerMoveDown)
        {
            playerRequestedWingsPitchAngleDeg = playerDownWingsPitchAngleDeg;
        }
        else if (vertValue >= inputDeadZone && playerMoveUp)
        {
            playerRequestedWingsPitchAngleDeg = playerUpWingsPitchAngleDeg;
        }
        else
        {
            playerRequestedWingsPitchAngleDeg = playerCenterWingsPitchAngleDeg;
        }

        if (playerCurrentWingsPitchAngleDeg > playerRequestedWingsPitchAngleDeg)
        {
            playerCurrentWingsPitchAngleDeg -= playerWingsPitchSpeedDegPerSec * Time.deltaTime;

            if (playerCurrentWingsPitchAngleDeg < playerRequestedWingsPitchAngleDeg)
            {
                playerCurrentWingsPitchAngleDeg = playerRequestedWingsPitchAngleDeg;
            }

            Vector3 playerWingsRotation = PlayerAvatarWings.transform.eulerAngles;
            playerWingsRotation.x = playerCurrentWingsPitchAngleDeg;
            PlayerAvatarWings.transform.eulerAngles = playerWingsRotation;
        }
        else if (playerCurrentWingsPitchAngleDeg < playerRequestedWingsPitchAngleDeg)
        {
            playerCurrentWingsPitchAngleDeg += playerBodyTurnSpeedDegPerSec * Time.deltaTime;

            if (playerCurrentWingsPitchAngleDeg > playerRequestedWingsPitchAngleDeg)
            {
                playerCurrentWingsPitchAngleDeg = playerRequestedWingsPitchAngleDeg;
            }

            Vector3 playerWingsRotation = PlayerAvatarWings.transform.eulerAngles;
            playerWingsRotation.x = playerCurrentWingsPitchAngleDeg;
            PlayerAvatarWings.transform.eulerAngles = playerWingsRotation;
        }
    }

    void HandlePlayerMovement(float horzValue, float vertValue)
    {
        if(!isPlayerMoving)
        {
            int centerTileX = (int)this.gameObject.transform.position.x;
            int centerTileY = (int)this.gameObject.transform.position.y;

            int tileLeft = (int)(this.gameObject.transform.position.x - 1.0f);
            int tileRight = (int)(this.gameObject.transform.position.x + 1.0f);
            int tileBelow = (int)(this.gameObject.transform.position.y - 1.0f);
            int tileAbove = (int)(this.gameObject.transform.position.y + 1.0f);

            if (horzValue <= -inputDeadZone && !ingameLogicComponent.IsTileBlocked(tileLeft, centerTileY))
            {
                isPlayerMoving = true;
                playerMoveLeft = true;
                playerTargetX = this.gameObject.transform.position.x - 1.0f;
           }
            else if (horzValue >= inputDeadZone && !ingameLogicComponent.IsTileBlocked(tileRight, centerTileY))
            {
                isPlayerMoving = true;
                playerMoveRight = true;
                playerTargetX = this.gameObject.transform.position.x + 1.0f;
           }
            else if (vertValue <= -inputDeadZone && !ingameLogicComponent.IsTileBlocked(centerTileX, tileBelow))
            {
                isPlayerMoving = true;
                playerMoveDown = true;
                playerTargetY = this.gameObject.transform.position.y - 1.0f;
           }
            else if (vertValue >= inputDeadZone && !ingameLogicComponent.IsTileBlocked(centerTileX, tileAbove))
            {
                isPlayerMoving = true;
                playerMoveUp = true;
                playerTargetY = this.gameObject.transform.position.y + 1.0f;
           }
        }
        else
        {
            if (playerMoveLeft)
            {
                Vector3 position = this.gameObject.transform.position;
                position.x -= playerSpeed;
                if (position.x < playerTargetX)
                {
                    position.x = playerTargetX;
                    playerMoveLeft = false;
                    isPlayerMoving = false;

                    this.gameObject.transform.position = position;

                    int centerTileX = (int)this.gameObject.transform.position.x;
                    int centerTileY = (int)this.gameObject.transform.position.y;
                    HandlePlayerCenterAction(centerTileX, centerTileY);
                }
                else
                {
                    this.gameObject.transform.position = position;
                }
            }
            else if (playerMoveRight)
            {
                Vector3 position = this.gameObject.transform.position;
                position.x += playerSpeed;
                if (position.x > playerTargetX)
                {
                    position.x = playerTargetX;
                    playerMoveRight = false;
                    isPlayerMoving = false;
                    isPlayerMoving = false;

                    this.gameObject.transform.position = position;

                    int centerTileX = (int)this.gameObject.transform.position.x;
                    int centerTileY = (int)this.gameObject.transform.position.y;
                    HandlePlayerCenterAction(centerTileX, centerTileY);
                }
                else
                {
                    this.gameObject.transform.position = position;
                }
            }
            else if (playerMoveDown)
            {
                Vector3 position = this.gameObject.transform.position;
                position.y -= playerSpeed;
                if (position.y < playerTargetY)
                {
                    position.y = playerTargetY;
                    playerMoveDown = false;
                    isPlayerMoving = false;
                    isPlayerMoving = false;

                    this.gameObject.transform.position = position;

                    int centerTileX = (int)this.gameObject.transform.position.x;
                    int centerTileY = (int)this.gameObject.transform.position.y;
                    HandlePlayerCenterAction(centerTileX, centerTileY);
                }
                else
                {
                    this.gameObject.transform.position = position;
                }
            }
            else if (playerMoveUp)
            {
                Vector3 position = this.gameObject.transform.position;
                position.y += playerSpeed;
                if (position.y > playerTargetY)
                {
                    position.y = playerTargetY;
                    playerMoveUp = false;
                    isPlayerMoving = false;
                    isPlayerMoving = false;

                    this.gameObject.transform.position = position;

                    int centerTileX = (int)this.gameObject.transform.position.x;
                    int centerTileY = (int)this.gameObject.transform.position.y;
                    HandlePlayerCenterAction(centerTileX, centerTileY);
                }
                else
                {
                    this.gameObject.transform.position = position;
                }
            }
        }
    }

    void HandlePlayerCenterAction(int x, int y)
    {
        ingameLogicComponent.RemoveDiggableTile(x, y);
        ingameLogicComponent.HandlePickupInTile(x, y);
        ingameLogicComponent.HandleKeyPickUp(x, y);
    }
}
