﻿using UnityEngine;
using UnityEngine.SceneManagement;
using System.Collections;
using System;
using UnityEngine.UI;

public enum TileType { None, Diggable, Wall, Rock, Door, Pickup, Key, Exit };

public class IngameSceneLogic : MonoBehaviour
{
    GameLogic gameLogicComponent = null;

    public GameObject Player;
    public bool AllowPlayerInput;
    public bool IngameReady;

    GameObject[] diggables;
    GameObject[] doors;
    GameObject[] walls;
    GameObject[] keys;
    GameObject[] pickups;
    GameObject[] exits;
    public GameObject[] rocks;

    public GameObject NearCamera;
    public GameObject FarCamera;

    public Text ScoreDisplayText;
    public Text ScoreDisplayShadowText;
    public Text TimeDisplayText;
    public Text TimeDisplayShadowText;
    public Text PickupDisplayText;
    public Text PickupDisplayShadowText;
    public Text LevelDisplayText;
    public Text LevelDisplayShadowText;

    public GameObject KeyImageEmpty;
    public GameObject KeyImageFull;
    bool isKeyPickedUp;

    float currentLevelCountdown;

    void Start()
    {
        IngameReady = false;

        GameObject gameLogicAnchor = GameObject.Find("GameLogicAnchor");
        if (gameLogicAnchor == null)
        {
            SceneManager.LoadScene("StartScene");
            return;
        }
        gameLogicComponent = gameLogicAnchor.GetComponent<GameLogic>();

        diggables = GameObject.FindGameObjectsWithTag("Diggable");
        doors = GameObject.FindGameObjectsWithTag("Door");
        walls = GameObject.FindGameObjectsWithTag("Wall");
        keys = GameObject.FindGameObjectsWithTag("Key");
        pickups = GameObject.FindGameObjectsWithTag("Pickup");
        exits = GameObject.FindGameObjectsWithTag("Exit");
        rocks = GameObject.FindGameObjectsWithTag("Rock");

        PreparePlayerSpawn();
        isKeyPickedUp = false;

        currentLevelCountdown = gameLogicComponent.GetCurrentLevelTime();

        HandlePickupUi();
        HandleLevelUi();
        HandleKeyUi();
        HandleScoreUi();

        AllowPlayerInput = true;
        IngameReady = true;
    }

    void Update()
    {
        if(!IngameReady)
        {
            return;
        }

        bool isBackPressed = Input.GetButtonUp("BackPressed");
        if (isBackPressed)
        {
            SceneManager.LoadScene("TitleScene");
            return;
        }

        bool toggleCamera = Input.GetButtonUp("ToggleCamera");
        HandleCameraToggle(toggleCamera);

        CheckForPlayerLevelExit();

        currentLevelCountdown -= Time.deltaTime;
        if(currentLevelCountdown <= 0.0f)
        {
            KillPlayer(this.gameObject);
        }
        HandleTimeUi();
    }

    void PreparePlayerSpawn()
    {
        GameObject[] playerSpawns = GameObject.FindGameObjectsWithTag("PlayerSpawn");

        int playerSpawnIndex = 0;
        if (playerSpawns.Length > 1)
        {
            playerSpawnIndex = UnityEngine.Random.Range(0, playerSpawns.Length - 1);
        }
        Player.transform.position = playerSpawns[playerSpawnIndex].transform.position;

        foreach (GameObject gameObject in playerSpawns)
        {
            gameObject.SetActive(false);
        }
    }

    void HandleCameraToggle(bool toggleCamera)
    {
        if (toggleCamera)
        {
            if (FarCamera.activeSelf)
            {
                NearCamera.SetActive(true);
                FarCamera.SetActive(false);
            }
            else
            {
                FarCamera.SetActive(true);
                NearCamera.SetActive(false);
            }
        }
    }

    public void HandlePickupInTile(int x, int y)
    {
        if (pickups.Length > 0)
        {
            for (int index = 0; index < pickups.Length; index++)
            {
                if (!pickups[index].activeSelf)
                {
                    continue;
                }

                Vector3 pickupPosition = pickups[index].transform.position;
                int pickupX = (int)pickupPosition.x;
                int pickupY = (int)pickupPosition.y;

                if (x == pickupX && y == pickupY)
                {
                    pickups[index].SetActive(false);
                    HandlePickupUi();

                    gameLogicComponent.PlayerScore += 25;
                    HandleScoreUi();

                    return;
                }
            }
        }
    }

    void HandlePickupUi()
    {
        int nosPickups = 0;

        if (pickups.Length > 0)
        {
            for (int index = 0; index < pickups.Length; index++)
            {
                if (pickups[index].activeSelf)
                {
                    nosPickups++;
                }
            }
        }

        PickupDisplayText.text = "MINERALS " + nosPickups.ToString();
        PickupDisplayShadowText.text = "MINERALS " + nosPickups.ToString();
    }

    public bool IsTileBlocked(int x, int y)
    {
        if (!isKeyPickedUp && doors.Length > 0)
        {
            for (int index = 0; index < doors.Length; index++)
            {
                if (!doors[index].activeSelf)
                {
                    continue;
                }

                Vector3 doorPosition = doors[index].transform.position;
                int doorX = (int)doorPosition.x;
                int doorY = (int)doorPosition.y;

                if (x == doorX && y == doorY)
                {
                    return true;
                }
            }
        }

        if (rocks.Length > 0)
        {
            for (int index = 0; index < rocks.Length; index++)
            {
                Vector3 rockPosition = rocks[index].transform.position;
                int rockX = (int)rockPosition.x;
                int rockY = (int)rockPosition.y;

                if (x == rockX && y == rockY)
                {
                    return true;
                }
            }
        }

        if (walls.Length > 0)
        {
            for (int index = 0; index < walls.Length; index++)
            {
                Vector3 wallPosition = walls[index].transform.position;
                int wallX = (int)wallPosition.x;
                int wallY = (int)wallPosition.y;

                if (x == wallX && y == wallY)
                {
                    return true;
                }
            }
        }

        return false;
    }

    public void RemoveDiggableTile(int x, int y)
    {
        if (diggables.Length > 0)
        {
            for (int index = 0; index < diggables.Length; index++)
            {
                if (!diggables[index].activeSelf)
                {
                    continue;
                }

                Vector3 diggablePosition = diggables[index].transform.position;
                int diggableX = (int)diggablePosition.x;
                int diggableY = (int)diggablePosition.y;

                if (x == diggableX && y == diggableY)
                {
                    diggables[index].SetActive(false);
                    return;
                }
            }
        }
    }

    public void HandleKeyPickUp(int x, int y)
    {
        if (keys.Length > 0)
        {
            for (int index = 0; index < keys.Length; index++)
            {
                if (!keys[index].activeSelf)
                {
                    continue;
                }

                Vector3 keyPosition = keys[index].transform.position;
                int keyX = (int)keyPosition.x;
                int keyY = (int)keyPosition.y;

                if (x == keyX && y == keyY)
                {
                    keys[index].SetActive(false);
                    isKeyPickedUp |= true;
                    HandleKeyUi();
                    return;
                }
            }
        }
    }

    void HandleKeyUi()
    {
        if (isKeyPickedUp)
        {
            KeyImageFull.SetActive(true);
            KeyImageEmpty.SetActive(false);
        }
        else
        {
            KeyImageEmpty.SetActive(true);
            KeyImageFull.SetActive(false);
        }
    }

    public TileType FindTileType(int x, int y)
    {
        if (diggables != null && diggables.Length > 0)
        {
            for (int index = 0; index < diggables.Length; index++)
            {
                if (!diggables[index].activeSelf)
                {
                    continue;
                }

                Vector3 position = diggables[index].transform.position;
                int posX = (int)position.x;
                int posY = (int)position.y;

                if (x == posX && y == posY)
                {
                    return TileType.Diggable;
                }
            }
        }

        if (keys != null && keys.Length > 0)
        {
            for (int index = 0; index < keys.Length; index++)
            {
                if (!keys[index].activeSelf)
                {
                    continue;
                }

                Vector3 position = keys[index].transform.position;
                int posX = (int)position.x;
                int posY = (int)position.y;

                if (x == posX && y == posY)
                {
                    return TileType.Key;
                }
            }
        }

        if (pickups != null && pickups.Length > 0)
        {
            for (int index = 0; index < pickups.Length; index++)
            {
                if (!pickups[index].activeSelf)
                {
                    continue;
                }

                Vector3 position = pickups[index].transform.position;
                int posX = (int)position.x;
                int posY = (int)position.y;

                if (x == posX && y == posY)
                {
                    return TileType.Pickup;
                }
            }
        }

        if (doors != null && doors.Length > 0)
        {
            for (int index = 0; index < doors.Length; index++)
            {
                Vector3 position = doors[index].transform.position;
                int posX = (int)position.x;
                int posY = (int)position.y;

                if (x == posX && y == posY)
                {
                    return TileType.Door;
                }
            }
        }

        if (exits != null && exits.Length > 0)
        {
            for (int index = 0; index < exits.Length; index++)
            {
                Vector3 position = exits[index].transform.position;
                int posX = (int)position.x;
                int posY = (int)position.y;

                if (x == posX && y == posY)
                {
                    return TileType.Exit;
                }
            }
        }

        if (rocks != null && rocks.Length > 0)
        {
            for (int index = 0; index < rocks.Length; index++)
            {
                Vector3 position = rocks[index].transform.position;
                int posX = (int)position.x;
                int posY = (int)position.y;

                if (x == posX && y == posY)
                {
                    return TileType.Rock;
                }
            }
        }

        if (walls != null && walls.Length > 0)
        {
            for (int index = 0; index < walls.Length; index++)
            {
                Vector3 position = walls[index].transform.position;
                int posX = (int)position.x;
                int posY = (int)position.y;

                if (x == posX && y == posY)
                {
                    return TileType.Wall;
                }
            }
        }

        return TileType.None;
    }

    public bool IsPlayerInTile(int x, int y)
    {
        Vector3 position = Player.transform.position;
        int posX = (int)position.x;
        int posY = (int)position.y;

        if (x == posX && y == posY)
        {
            return true;
        }

        return false;
    }

    void CheckForPlayerLevelExit()
    {
        Vector3 position = Player.transform.position;
        int x = (int)position.x;
        int y = (int)position.y;

        TileType currentPlayerTile = FindTileType(x, y);
        if (currentPlayerTile == TileType.Exit && isKeyPickedUp)
        {
            if (gameLogicComponent.MoveToNextLevel())
            {
                SceneManager.LoadScene(gameLogicComponent.GetCurrentLevelName());
            }
            else
            {
                SceneManager.LoadScene("GameWonScene");
            }
            return;
        }
    }

    void HandleLevelUi()
    {
        string currentLevelText = gameLogicComponent.CurrentLevel.ToString();

        LevelDisplayText.text = "LEVEL " + currentLevelText;
        LevelDisplayShadowText.text = "LEVEL " + currentLevelText;
    }

    public void KillPlayer(GameObject gameObject)
    {
        SceneManager.LoadScene("GameLostScene");
        return;
    }

    void HandleTimeUi()
    {
        int timeMinutes = (int)(currentLevelCountdown / 60.0f);
        int timeSeconds = (int)(currentLevelCountdown % 60.0f);

        string currentTimeMinutesText = timeMinutes.ToString();
        if(timeMinutes <= 9)
        {
            currentTimeMinutesText = "0" + currentTimeMinutesText;
        }
        string currentTimeSecondsText = timeSeconds.ToString();
        if(timeSeconds <= 9)
        {
            currentTimeSecondsText = "0" + currentTimeSecondsText;
        }

        TimeDisplayText.text = "TIME " + currentTimeMinutesText + ":" + currentTimeSecondsText;
        TimeDisplayShadowText.text = "TIME " + currentTimeMinutesText + ":" + currentTimeSecondsText;
    }

    void HandleScoreUi()
    {      
        int score = gameLogicComponent.PlayerScore;
        if(score > 99999)
        {
            score = 99999;
        }

        string scoreText = string.Empty;

        int score4 = score / 10000;
        score = score - (score4 * 10000);
        scoreText += score4.ToString();

        int score3 = score / 1000;
        score = score - (score3 * 1000);
        scoreText += score3.ToString();

        int score2 = score / 100;
        score = score - (score2 * 100);
        scoreText += score2.ToString();

        int score1 = score / 10;
        score = score - (score1 * 10);
        scoreText += score1.ToString();

        int score0 = score;
        scoreText += score0.ToString();

        ScoreDisplayText.text = "SCORE " + scoreText;
        ScoreDisplayShadowText.text = "SCORE " + scoreText;
    }
}
